% load the measured cam data
% this is the displacement of the follower
load cam.dat
% copy the follower displacement in the row vector y
y = cam.';

% plot the follower displacement y as a funtion of the angle p
% generate p, equidistance from 0 to 2pi
n = length(y);
p = linspace(0,2*pi,n+1);
p = p(1:n);
f1 = figure;
plot(p./pi,y,'-b.')
ylabel('Displ y [mm]')
xlabel('\phi/\pi')
title('Measured data')
grid

%% now do cubic interpolate with another number of points
u = linspace(0,2*pi,100);
u = u(1:end-1);
v = splinetx(p,y,u);
figure(f1);
hold on
plot(u./pi,v,'-r.');
title('Measured and Interpolated data')

%% next show the firts and second order derivatives

[v,dv,ddv] = mysplinetx(p,y,u);
f2 = figure;
subplot(3,1,1)
plot(u./pi,v);
xlabel('\phi/\pi')
ylabel('v')
title('Interpolated values of y, y'' and y'''' with splinetx') 
grid
subplot(3,1,2)
plot(u./pi,dv);
xlabel('\phi/\pi')
ylabel('y''')
grid
subplot(3,1,3)
plot(u./pi,ddv);
xlabel('\phi/\pi')
ylabel('y''''')
grid
set(f2,'Position',[100 100 400 800])

%% transform measured data to fourier form
Y = fft(y);
n = length(Y);
% and plot the absolute value of the amplitudes
figure
plot(abs(Y),'.')
grid

%% get the A_j and B_j from the fourier transform
% as in y=Sumj[Aj*cos(2*pi*j*f)+Bj*sin(2*pi*j*f)]
[A,B]=f2ab(Y);
% only consider the first nharm harmonics
nharm = 10;
harm = 0:nharm-1;
A = A(1:nharm);
B = B(1:nharm);

% and plot these harmonic contributions
figure
plot(harm,A,'o')
grid
hold on
plot(harm,B,'*')
xlabel('j^{th} Harmonic')
ylabel('Amplitudes, a_j and b_j')
legend('a_j','b_j')
title('Harmonics from y=\Sigma [a_j cos(j \phi ) + b_j sin(j \phi)], \phi=0:2\pi')

%% reconstruct y from the A and B fourier form
% use u for the angle phi
yf = zeros(size(u));
dyf = yf;
ddyf = yf;
for j=0:nharm-1
  yf = yf + A(j+1).*cos(j.*u)+B(j+1).*sin(j.*u);
  dyf = dyf - j*A(j+1).*sin(j.*u)+j*B(j+1).*cos(j.*u);
  ddyf = ddyf - j^2*A(j+1).*cos(j.*u)-j^2*B(j+1).*sin(j.*u);
end
% and plot the result in the plot with cubic interpolated results
figure(f2)
subplot(3,1,1)
hold on
plot(u./pi,yf,'r');
subplot(3,1,2)
hold on
plot(u./pi,dyf,'r');
subplot(3,1,3)
hold on
plot(u./pi,ddyf,'r');



